@@ -17,12 +17,13 @@
 #   License along with this library; if not, write to the Free Software
 #   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import gi
+gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk, GObject, Gio, GdkPixbuf, Gdk, GMenu, GLib
 import cgi
-import codecs
 import os
 import gettext
-import subprocess
+import shutil
 
 from Alacarte import config
 gettext.bindtextdomain(config.GETTEXT_PACKAGE, config.localedir)
@@ -30,22 +31,15 @@ gettext.textdomain(config.GETTEXT_PACKAGE)
 
 _ = gettext.gettext
 from Alacarte.MenuEditor import MenuEditor
+from Alacarte.ItemEditor import LauncherEditor, DirectoryEditor
 from Alacarte import util
 
 class MainWindow(object):
-    timer = None
-    #hack to make editing menu properties work
-    edit_pool = []
-
-    def __init__(self, datadir, version):
-        self.file_path = datadir
-        self.version = version
-        self.editor = MenuEditor()
-        self.editor.tree.connect("changed", self.menuChanged)
+    def __init__(self):
         Gtk.Window.set_default_icon_name('alacarte')
         self.tree = Gtk.Builder()
         self.tree.set_translation_domain(config.GETTEXT_PACKAGE)
-        self.tree.add_from_file(os.path.join(self.file_path, 'alacarte.ui'))
+        self.tree.add_from_file(os.path.join(config.pkgdatadir, 'alacarte.ui'))
         self.tree.connect_signals(self)
         self.setupMenuTree()
         self.setupItemTree()
@@ -57,7 +51,19 @@ class MainWindow(object):
         accelgroup = Gtk.AccelGroup()
         keyval, modifier = Gtk.accelerator_parse('F1')
         accelgroup.connect(keyval, modifier, Gtk.AccelFlags.VISIBLE, self.on_help_button_clicked)
-        self.tree.get_object('mainwindow').add_accel_group(accelgroup)
+
+        self.main_window = self.tree.get_object('mainwindow')
+        self.main_window.add_accel_group(accelgroup)
+
+        self.editor = None
+
+    def setMenuBasename(self, menu_basename):
+        if self.editor is not None:
+            self.editor.tree.disconnect(self.menuChangedId)
+
+        self.editor = MenuEditor(menu_basename)
+        self.menuChangedId = self.editor.tree.connect("changed", self.menuChanged)
+        self.menuChanged()
 
     def run(self):
         self.loadMenus()
@@ -202,8 +208,6 @@ class MainWindow(object):
     def loadMenu(self, iters, parent=None):
         for menu, show in self.editor.getMenus(parent):
             name = cgi.escape(menu.get_name())
-            if not show:
-                name = "<small><i>%s</i></small>" % (name,)
 
             icon = util.getIcon(menu)
             iters[menu] = self.menu_store.append(iters[parent], (icon, name, menu))
@@ -223,33 +227,11 @@ class MainWindow(object):
                 assert False, 'should not be reached'
 
             name = cgi.escape(name)
-            if not show:
-                name = "<small><i>%s</i></small>" % (name,)
 
             self.item_store.append((show, icon, name, item))
 
-    #this is a little timeout callback to insert new items after
-    #gnome-desktop-item-edit has finished running
-    def waitForNewItemProcess(self, process, parent_id, file_path):
-        if process.poll() is not None:
-            if os.path.isfile(file_path):
-                self.editor.insertExternalItem(os.path.split(file_path)[1], parent_id)
-            return False
-        return True
-
-    def waitForNewMenuProcess(self, process, parent_id, file_path):
-        if process.poll() is not None:
-            if os.path.isfile(file_path):
-                self.editor.insertExternalMenu(os.path.split(file_path)[1], parent_id)
-            return False
-        return True
-
-    #this callback keeps you from editing the same item twice
-    def waitForEditProcess(self, process, file_path):
-        if process.poll() is not None:
-            self.edit_pool.remove(file_path)
-            return False
-        return True
+    def on_delete_event(self, widget, event):
+        self.quit()
 
     def on_new_menu_button_clicked(self, button):
         menu_tree = self.tree.get_object('menu_tree')
@@ -261,8 +243,9 @@ class MainWindow(object):
         else:
             parent = menus[iter][2]
         file_path = os.path.join(util.getUserDirectoryPath(), util.getUniqueFileId('alacarte-made', '.directory'))
-        process = subprocess.Popen(['gnome-desktop-item-edit', file_path], env=os.environ)
-        GObject.timeout_add(100, self.waitForNewMenuProcess, process, parent.get_menu_id(), file_path)
+
+        editor = DirectoryEditor(self.main_window, file_path)
+        editor.run()
 
     def on_new_item_button_clicked(self, button):
         menu_tree = self.tree.get_object('menu_tree')
@@ -274,8 +257,9 @@ class MainWindow(object):
         else:
             parent = menus[iter][2]
         file_path = os.path.join(util.getUserItemPath(), util.getUniqueFileId('alacarte-made', '.desktop'))
-        process = subprocess.Popen(['gnome-desktop-item-edit', file_path], env=os.environ)
-        GObject.timeout_add(100, self.waitForNewItemProcess, process, parent.get_menu_id(), file_path)
+
+        editor = LauncherEditor(self.main_window, file_path)
+        editor.run()
 
     def on_new_separator_button_clicked(self, button):
         item_tree = self.tree.get_object('item_tree')
@@ -314,19 +298,24 @@ class MainWindow(object):
         if isinstance(item, GMenu.TreeEntry):
             file_path = os.path.join(util.getUserItemPath(), item.get_desktop_file_id())
             file_type = 'Item'
+            Editor = LauncherEditor
         elif isinstance(item, GMenu.TreeDirectory):
             file_path = os.path.join(util.getUserDirectoryPath(), os.path.split(item.get_desktop_file_path())[1])
             file_type = 'Menu'
+            Editor = DirectoryEditor
 
+        copied = False
         if not os.path.isfile(file_path):
-            data = open(item.get_desktop_file_path()).read()
-            with codecs.open(file_path, 'w', 'utf8') as f:
-                f.write(data)
+            shutil.copy(item.get_desktop_file_path(), file_path)
+            copied = True
 
-        if file_path not in self.edit_pool:
-            self.edit_pool.append(file_path)
-            process = subprocess.Popen(['gnome-desktop-item-edit', file_path], env=os.environ)
-            GObject.timeout_add(100, self.waitForEditProcess, process, file_path)
+        editor = Editor(self.main_window, file_path)
+        editor.connect('response', self.on_editor_response, file_path if copied else None)
+        editor.run()
+
+    def on_editor_response(self, editor, modified, file_path):
+        if not modified and file_path is not None:
+            os.remove(file_path)
 
     def on_menu_tree_cursor_changed(self, treeview):
         selection = treeview.get_selection()
@@ -453,3 +442,13 @@ class MainWindow(object):
 
     def quit(self):
         Gtk.main_quit()
+
+def main():
+    if len(sys.argv) > 1:
+        basename = sys.argv[1]
+    else:
+        basename = None
+
+    app = MainWindow()
+    app.setMenuBasename(basename)
+    app.run()
